local _, core = ...

local module = core.NewModule("Status Online")

 --[[ -------- Locals ------- ]]--

local listIsDirty

local assumeMissingAreOnline -- If true, treat players not found in guild/friend list as being online

local sortersModule

local listFriends = {}
local listGuild = {}
local altLookup
--local listChannels = setmetatable({}, meta) -- Need to figure out how to do this easily...

local libAlts = LibStub("LibAlts-1.0")

local timerFrame = CreateFrame("frame") -- Minimal timer to minimally cut down like one whole list update call, maybe. Like wow

local ContextMenuEnableCheck
local ContextMenuFunc

 --[[ -------- Timer -------- ]]--

do
	local totalElapsed = 0
	timerFrame.Start, timerFrame.Stop = timerFrame.Show, timerFrame.Hide -- Useless fluff, I know
	timerFrame:SetScript("OnUpdate", function(self, elapsed)
		totalElapsed = totalElapsed + elapsed
		if totalElapsed >= 2 then
			totalElapsed = 0
			self:Stop()
			if listIsDirty then
				listIsDirty = nil
				core.UpdateOpenList() -- This is a bit heavy call I guess, but I don't want to recolorize all the entries myself
			end
		end
	end)
end

 --[[ ----------------------- ]]--

local function Capitalize(str)
	if not str then return end
	return str:sub(1, 1):upper()..str:sub(2)
end

local buttons = {}
local function DimButton(button)
		local buttonName = button:GetName()
		local name = _G[buttonName.."Name"]
		buttons[1], buttons[2], buttons[3] = _G[buttonName.."Class"], _G[buttonName.."Status"], sortersModule and _G[buttonName.."Role"] or nil
		name:SetTextColor(0.6, 0.6, 0.6, 0.65)
		local r, g, b
		for _, button in ipairs(buttons) do
			r, g, b = button:GetTextColor()
			button:SetTextColor(r, g, b, 0.5)
		end
end

local function IsOnline(name)
	return listGuild[name] or listFriends[name] or (UnitInRaid(name) and UnitIsConnected(name)) or (UnitInParty(name) and UnitIsConnected(name)) --[[ or listChannels[name] ]]
end

local function FindPlayer(name)
	local online = IsOnline(name)
	local main
	if not online then -- I kinda dislike this, it seems like a lot of stuff to do on each offline player in the list, each time the list update fires (which it does pretty often too), but can't figure out a more elegant way right now
		main = libAlts:IsMain(name) and name or libAlts:GetMain(name)
		if main and main ~= name then
			online = IsOnline(main) and main
		end
		if not online and main then
			local t = libAlts.Alts[main:lower()] -- This is probably not a good idea, but why is the LibAlts:GetAlts() unpacking the return value for me? That's no good. O_o
			if t and #t > 0 then
				for i, alt in pairs(t) do
					if alt ~= name then
						alt = Capitalize(alt)
						online = IsOnline(alt) and alt
						if online then break end
					end
				end
			end
		end
	end
	return online
end

function module:FRIENDLIST_UPDATE()
	for i = 1, GetNumFriends() do
		local name, _, _, _, online = GetFriendInfo(i)
		local status
		if (assumeMissingAreOnline and not online) or (not assumeMissingAreOnline and online) then
			status = true
		end
		if not listIsDirty and listFriends[name] ~= status then
			listIsDirty = true
			altLookup = nil
			timerFrame:Start()
		end
		listFriends[name] = status
	end
end

function module:GUILD_ROSTER_UPDATE()
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(i)
		local status
		if (assumeMissingAreOnline and not online) or (not assumeMissingAreOnline and online) then
			status = true
		end
		if not listIsDirty and listGuild[name] ~= status then
			listIsDirty = true
			altLookup = nil
			timerFrame:Start()
		end
		listGuild[name] = status
	end
end

-- If a person is online but on an alt, note this in the tooltip when mouseovering the entry in the list
local function OnEnter(frame)
	if not altLookup then return end
	local name = CalendarEventGetInvite(frame.inviteIndex)
	local alt = altLookup[name]
	if alt then
		if not GameTooltip:GetOwner() then
			GameTooltip:SetOwner(frame, "ANCHOR_TOPRIGHT");
		else
			GameTooltip:AddLine(" ")
		end
		GameTooltip:AddLine("(On his/her alt |cFFFFD100"..alt.."|r)", 0.6, 0.6, 0.6)
		GameTooltip:Show()
	end
end

-- Dim buttons for the offline people. This fires after the original Calendar***EventInviteListScrollFrame_Update which changes the colors for the buttons, so no need to "undim" them
local function UpdateInviteListButtons(_, frame)
	for _, button in ipairs(frame.buttons) do
		local inviteIndex = button.inviteIndex
		if inviteIndex then
			local name = CalendarEventGetInvite(inviteIndex)
			local found = FindPlayer(name)
			if type(found) == "string" then
				if not altLookup then altLookup = {} end
				altLookup[name] = Capitalize(found) -- Remember which alt this person is logged on on. I suck at naming variables :-(
				if not button:GetScript("OnEnter") then
					button:SetScript("OnEnter", OnEnter)
				end
			end
			if (assumeMissingAreOnline and found) or (not assumeMissingAreOnline and not found) then
				DimButton(button)
			end
		end
	end
end

function module.OnInitialize()
	if core.modules["Sorters"] then
		sortersModule = core.modules["Sorters"]
	end

	core.RegisterCallback(module, "Post_Update", UpdateInviteListButtons)

	-- Need to refresh things if people go online/offline, hopefully these catch all things properly.
	core.RegisterCallback(module, "CalendarShow", function()
		module:RegisterEvent("GUILD_ROSTER_UPDATE")
		module:RegisterEvent("FRIENDLIST_UPDATE")
		module:GUILD_ROSTER_UPDATE()
		module:FRIENDLIST_UPDATE()
	end)       
	core.RegisterCallback(module, "CalendarHide", function()
		module:UnregisterEvent("GUILD_ROSTER_UPDATE")
		module:UnregisterEvent("FRIENDLIST_UPDATE")
		timerFrame:Stop()
	end)
	module:RegisterEvent("GUILD_ROSTER_UPDATE")
	module:RegisterEvent("FRIENDLIST_UPDATE")
	module:GUILD_ROSTER_UPDATE()
	module:FRIENDLIST_UPDATE()
	GuildRoster()

	-- Post-hook to add alt-info to the existing tooltips
	module:RawHook("CalendarEventInviteListButton_OnEnter", function(frame, ...)
		module.hooks.CalendarEventInviteListButton_OnEnter(frame, ...)
		OnEnter(frame)
	end, true)

	-- Add whisper-to-alt for easy yelling at n00bs who're still on an alt at raid start! >:-(
	core.NewContextMenuEntry({
		text = "Whisper alt",
		func = ContextMenuFunc,
		enable = ContextMenuEnableCheck,
	})
end

 --[[ ----- Context menu ---- ]]--

-- Only enable for the players that are on an alt
function ContextMenuEnableCheck(inviteIndex)
	return altLookup and altLookup[(CalendarEventGetInvite(inviteIndex))]
end

-- Open whisper to the alt, and then let 'em have it! This'll be the last time that slacker misses a raid invite...
function ContextMenuFunc()
	if not altLookup then return end
	local name = CalendarEventGetInvite(core.ContextMenuGetInviteButton().inviteIndex)
	local alt = altLookup[name]
	if alt then
		ChatFrame_SendTell(alt)
	end
end

--[[
	Some of the channel functions I found
local channel, header, collapsed, channelNumber, count, active, category = GetChannelDisplayInfo(id)
local name, owner, moderator, muted, active, enabled = GetChannelRosterInfo(id, rosterIndex)
SetSelectedDisplayChannel(id)
CHANNEL_ROSTER_UPDATE(channelId)
]]