--[[
	Project version: v1.4.1
	Project revision: 35

	File revision: 33
]]
local name, core = ...
CalendarExtras = core

core.modules = {}

local tabs = {}
local PositionTabs

local contextMenuEntries = {}

 --[[ ------------------------ ]]--

LibStub("AceEvent-3.0"):Embed(core)
LibStub("AceHook-3.0"):Embed(core)

core.callbacks = LibStub("CallbackHandler-1.0"):New(core)

function core.NewModule(name)
	local module = {}
	core.modules[name] = module
	LibStub("AceHook-3.0"):Embed(module)
	LibStub("AceEvent-3.0"):Embed(module)
	return module
end

function core.UpdateOpenList()
	if CalendarViewEventFrame:IsShown() then
		core:CalendarViewEventInviteListScrollFrame_Update()
	elseif CalendarCreateEventFrame:IsShown() then
		core:CalendarCreateEventInviteListScrollFrame_Update()
	end
end

function core:ADDON_LOADED(event, addon)
	if addon == "CalendarExtras" then
		self:UnregisterEvent("ADDON_LOADED")
		self.ADDON_LOADED = nil

		if not CalendarExtrasDB then CalendarExtrasDB = {} end
		self.db = CalendarExtrasDB

		for _, module in pairs(self.modules) do
			if module.OnInitialize then
				module.OnInitialize()
				module.OnInitialize = nil
			end
		end
		self:HookScript(CalendarViewEventFrame, "OnShow", PositionTabs)
		self:HookScript(CalendarCreateEventFrame, "OnShow", PositionTabs)
	end
end
core:RegisterEvent("ADDON_LOADED")

 --[[ ------- Callbacks ------ ]]--

-- Three callbacks, "OpenEvent" for before list update when opening event and "[Pre|Post]_Update" for before|after list update

local openingEvent

local function Opening()
	openingEvent = true
end
core:Hook("CalendarOpenEvent", Opening, true)
core:Hook("CalendarNewEvent", Opening, true) 

function core:CalendarViewEventInviteListScrollFrame_Update(...)
	local f = CalendarViewEventInviteListScrollFrame
	if openingEvent then
		openingEvent = nil
		self.callbacks:Fire("OpenEvent", f)
	end
	self.callbacks:Fire("Pre_Update", f)
	self.hooks.CalendarViewEventInviteListScrollFrame_Update(...)
	self.callbacks:Fire("Pre_Post_Update", f) -- Sorters' listupdate needs to run first so it can fix the inviteindices, but callbacks seem to come in arbitrary order, lets just give it it's own callback for now... getting tired of having to fine-tune the firing-order of all these billion functions or the whole thing collapses -_-
	self.callbacks:Fire("Post_Update", f)
end
core:RawHook("CalendarViewEventInviteListScrollFrame_Update", true)
core:RawHook(CalendarViewEventInviteList.scrollFrame, "update", "CalendarViewEventInviteListScrollFrame_Update", true)

function core:CalendarCreateEventInviteListScrollFrame_Update(...)
	local f = CalendarCreateEventInviteListScrollFrame
	if openingEvent then
		openingEvent = nil
		self.callbacks:Fire("OpenEvent", f)
	end
	self.callbacks:Fire("Pre_Update", f)
	self.hooks.CalendarCreateEventInviteListScrollFrame_Update(...)
	self.callbacks:Fire("Pre_Post_Update", f)
	self.callbacks:Fire("Post_Update", f)
end
core:RawHook("CalendarCreateEventInviteListScrollFrame_Update", true)
core:RawHook(CalendarCreateEventInviteList.scrollFrame, "update", "CalendarCreateEventInviteListScrollFrame_Update", true)

core:HookScript(CalendarFrame, "OnShow", function(frame)
	core.callbacks:Fire("CalendarShow")
end)       
core:HookScript(CalendarFrame, "OnHide", function(frame)
	core.callbacks:Fire("CalendarHide")
end)

 --[[ --------- Tabs --------- ]]--

local function tabOnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
	if self.tooltipText and self.frameToOpen and not self.frameToOpen:IsShown() or not self.frameToOpen then
		GameTooltip:SetText(self.tooltipText, 1.0, 0.82, 0)
	end
end

local function tabOnHide(self)
	if self.frameToOpen then self.frameToOpen:Hide() end
end

local function tabOnClick(self, button)
	if self.frameToOpen and self.frameToOpen:IsShown() then
		self.frameToOpen:Hide()
	elseif self.frameToOpen then
		self:LockHighlight()
		self.frameToOpen:Show()
	end
	if self.func then self.func() end
	GameTooltip:Hide()
end

local function frameToOpenOnHide(self)
	self.parentTab:UnlockHighlight()
end

function PositionTabs(frame)
	for i, tab in ipairs(tabs) do
		tab:ClearAllPoints()
		tab:SetParent(frame)
		if i == 1 then
			tab:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 10, 5)
		else
			tab:SetPoint("TOPLEFT", tabs[i-1], "TOPRIGHT", 5, 0)
		end
		tab:SetFrameStrata("MEDIUM")
		tab:Show()
	end
end

-- local tab = core.NewTab("CalendarExtrasTestTab", "Test", "Opens frameToOpen", frameXyz, function() doSomethingFancy end)
function core.NewTab(text, tooltipText, frameToOpen, func)
	local name = "CalendarExtras"..text.."Tab"
	local tab = CreateFrame("Button", name, CalendarFrame, "CharacterFrameTabButtonTemplate")
	tab.tooltipText = tooltipText
	tab.frameToOpen = frameToOpen
	tab.func = func
	table.insert(tabs, tab)

	tab:SetText(text)
	PanelTemplates_TabResize(tab, 10)
	_G[name.."HighlightTexture"]:SetWidth(tab:GetTextWidth() + 20)

	tab:SetScript("OnEnter", tabOnEnter)
	tab:SetScript("OnLeave", GameTooltip_Hide)
	tab:SetScript("OnShow", nil) -- Clear the default OnShow of this template
	tab:SetScript("OnHide", tabOnHide)
	tab:SetScript("OnClick", tabOnClick)

	if frameToOpen then
		frameToOpen.parentTab = tab
		core:HookScript(frameToOpen, "OnHide", frameToOpenOnHide)
	end
end

 --[[ ----- Context menus ---- ]]--

local subMenus = {}

--[[
local entryTable = {
		text = "Entry 1 text",
		subMenu = "subMenu1Name" or nil,
		func = FancyFunction1 or nil,
		onlyInMenuX = "create" or "view" or nil,
		enable = checkEnabledStateFunc, -- local function checkEnabledStateFunc(inviteIndex) return true end
} or {
	{text = "Entry1"},
	{text = "Entry2"},
	etc...
}
core.NewContextMenuEntry(entryTable)
]]
function core.NewContextMenuEntry(entryTable)
	if entryTable.text then
		contextMenuEntries[#contextMenuEntries + 1] = {entryTable}
	else
		local t = {}
		for k, v in ipairs(entryTable) do
			t[#t + 1] = v
		end
		contextMenuEntries[#contextMenuEntries + 1] = t
	end
	if entryTable.subMenu then
		subMenus[#subMenus + 1] = entryTable.subMenu
	end
end

-- A bit redundant maybe...
function core.ContextMenuGetInviteButton()
	return CalendarContextMenu.inviteButton
end

local function AddEntriesToMenu(menu, originatingFrame)
	local groupsAdded = 0
	for k, group in ipairs(contextMenuEntries) do
		if groupsAdded > 0 then
			UIMenu_AddButton(menu)
		end
		local entriesAddedToGroup = 0
		for _, entry in ipairs(group) do
			if not entry.onlyInMenuX or entry.onlyInMenuX == originatingFrame then
				if (entry.enable and entry.enable(menu.inviteButton.inviteIndex)) or not entry.enable then
					UIMenu_AddButton(menu, entry.text, nil, entry.func, entry.subMenu)
					entriesAddedToGroup = entriesAddedToGroup + 1
				end
			end
		end
		if entriesAddedToGroup > 0 then
			groupsAdded = groupsAdded + 1
		end
	end
end

-- Hook the contex-menu generator to add in our own menu entries
function core:CalendarCreateEventInviteContextMenu_Initialize(...)
	self.hooks.CalendarCreateEventInviteContextMenu_Initialize(...)

	local menu = ...

	AddEntriesToMenu(menu, "create")

	return UIMenu_FinishInitializing(menu)
end
core:RawHook("CalendarCreateEventInviteContextMenu_Initialize", true)

-- Same for ViewEvent. This doesn't normally open as there is no contextmenu for ViewEvent by default, so lets just replace the whole function with our own
function core:CalendarViewEventInviteContextMenu_Initialize(...)
	local menu, inviteButton = ...

	UIMenu_Initialize(menu, "view")
	CalendarInviteContextMenu_UnlockHighlights()
	menu.inviteButton = inviteButton
	inviteButton:LockHighlight()

	AddEntriesToMenu(menu)

	return UIMenu_FinishInitializing(menu)
end
core:RawHook("CalendarViewEventInviteContextMenu_Initialize", true)

-- Hook the viewevent onclick function to force our right-click menu to always open
function core:CalendarViewEventInviteListButton_OnClick(self, button)
	if button == "LeftButton" then
		CalendarContextMenu_Hide()
	elseif button == "RightButton" then
		if CalendarContextMenu.inviteButton ~= self then
			CalendarContextMenu_Show(self, CalendarViewEventInviteContextMenu_Initialize, "cursor", 3, -3, self)
		else
			CalendarContextMenu_Toggle(self, CalendarViewEventInviteContextMenu_Initialize, "cursor", 3, -3, self)
		end
	end
	PlaySound("igMainMenuOptionCheckBoxOn")
end
core:RawHook("CalendarViewEventInviteListButton_OnClick", true)

-- Hook to hide any submenus
core:Hook("CalendarContextMenu_OnHide", function()
	for k, v in ipairs(subMenus) do
		v:Hide()
	end
end, true)